import s3fs
import slate3k
import logging
from tqdm import tqdm
import os
import pandas as pd
import json
from typing import Tuple, Dict, Optional


logger = logging.getLogger(__name__)


class DocumentScraper:
    """
    This class has methods for scraping all the files with '.pdf' extension within a given folder, cleaning the text,
    and generating a pd.DataFrame where:
        - each column contains the text of a separate pdf file (column name is pdf title without extension), and
        - each row within a column contains the text of one page within that pdf.
    If different pdf files have different number of pages, any empty rows at the bottom of a column are filled with nan.
    """
    def __init__(self, pdf_folder: str, json_filename: Optional[str] = None) -> None:
        """
        :param pdf_folder: path to folder containing pdf files to be scraped
        :param json_filename: full path of the json file created by the module json_creator.py. This json file
               contains dictionary of words to replace (e.g. Dr. --> Dr), used for text cleaning. Defaults to None, in
               which case no ad-hoc text cleaning will be performed.
        """
        self.pdf_folder = pdf_folder
        self.open_json = self._read_config(json_filename)

    @staticmethod
    def _read_config(json_filename: Optional[str]) -> Dict[str, str]:
        """
        :param json_filename: json filename to be deserialized.
        :return: the dictionary from json object. If json_filename is None, and empty dictionary will be returned.
        """
        if json_filename is None:
            logger.warning('No .json file for text cleaning was provided. Ad-hoc text cleaning will not be performed.')
            return dict()
        logger.info(f'Reading {json_filename} file for text cleaning.')
        assert '.json' in json_filename, 'The json_filename provided does not correspond to a .json file.'
        with open(json_filename, 'r') as file:
            return json.load(file)

    def _text_to_series_of_pages(self, pdf_name: str) -> Tuple[pd.Series, int]:
        """
        :param pdf_name: full name of pdf (including .pdf extension) to be scraped and converted into a pd.Series
        :return: document_series: a pd.Series where each row contains the text of one pdf page.
                 num_pages: int, the number of pages of the input pdf file
        """
        assert pdf_name.endswith('.pdf'), 'Input file is not in .pdf format. The file cannot be processed.'
        document_series = pd.Series()
        try:
            pdf = open(os.path.join(self.pdf_folder, pdf_name), 'rb')
        except FileNotFoundError:  # check if the path corresponds to an S3 bucket
            try:
                pdf = s3fs.S3FileSystem().open(os.path.join(self.pdf_folder, pdf_name), 'rb')
            except FileNotFoundError as err:
                raise FileNotFoundError(
                    f"{err}. We also tried to look for an S3 bucket path but could not find any. Other types of cloud "
                    f"storage are not natively supported by pdf2embeddings."
                )
        pdf_reader = slate3k.PDF(pdf)
        num_pages = len(pdf_reader)
        for i, page in enumerate(pdf_reader):
            logger.debug(f'Reading page {i+1} of PDF file {pdf_name}')
            page_text = self._clean_text(page)
            page_series = pd.Series(page_text)
            document_series = document_series.append(page_series, ignore_index=True)
        pdf.close()

        return document_series, num_pages

    def _clean_text(self, text: str) -> str:
        """
        :param text: the text to be cleaned. This replaces certain words based on the dict self.open_json
        :return: text: the cleaned text.
        """
        for k, v in self.open_json.items():
            text = text.replace(k, v)
        text = text.strip()
        return text

    def document_corpus_to_pandas_df(self) -> pd.DataFrame:
        """
        This method can be called by the user to generate the final pd.DataFrame as described in class docstring.
        :return: df: a pd.DataFrame. See class docstring.
        """
        df = pd.DataFrame()
        pdf_list = [pdf for pdf in os.listdir(self.pdf_folder) if pdf.endswith('.pdf')]  # excluding non .pdf files
        not_pdf_list = [pdf for pdf in os.listdir(self.pdf_folder) if not pdf.endswith('.pdf')]
        if len(not_pdf_list) > 0:
            logger.warning(
                f'\nThe following files were present in the directory {self.pdf_folder}, but were not scraped as they '
                f'are not in .pdf format: \n{not_pdf_list}'
            )
        logger.info('Starting scraping PDFs...')
        for i, file in enumerate(tqdm(sorted(pdf_list))):
            # sorted is so pdfs are extracted in alphabetic order, and to make testing more robust.
            series, num_pages = self._text_to_series_of_pages(file)
            logger.info(f"Reading PDF file {i + 1} out of {len(pdf_list)}: \"{file}\", number of pages: {num_pages}")
            if isinstance(series, pd.Series):
                series.rename(file.replace('.pdf', ''), inplace=True)
                df = pd.concat([df, series], axis=1)
        return df
    



import nltk
import pandas as pd
import numpy as np
import pickle
import logging
from scipy.spatial.distance import cdist
from gensim.models import Word2Vec
from allennlp.commands.elmo import ElmoEmbedder
from sentence_transformers import SentenceTransformer
from sklearn.feature_extraction.text import TfidfVectorizer
from typing import Tuple, Optional, Union


logger = logging.getLogger(__name__)


def query_embeddings(
        user_input: str,
        trained_df: Union[str, pd.DataFrame],
        expected_embeddings_colname: str,
        model_name: str,
        model: Optional[Union[str, Word2Vec, ElmoEmbedder, SentenceTransformer]] = None,
        expected_sentence_colname: str = 'sentence',
        distance_metric: str = 'cosine',
        metric_colname: str = 'metric_distance',
        tfidf_vectorizer: Optional[Union[str, TfidfVectorizer]] = None
) -> Tuple[np.ndarray, pd.DataFrame]:
    """
    Takes a user input search, embeds it using the model of choice (available: Word2Vec with tf-idf weighting, Word2Vec
    without tf-idf weighting, ELMo and BERT), and compares this embedding against all the embeddings of a given corpus
    of sentences, calculating the similarity score. Embeddings must all be sentence-level.
    NOTE: this uses all models in testing mode. No training is performed and models are not updated to include user
    queries.
    :param user_input: the user search query
    :param trained_df: a pd.DataFrame, or a string pointing to a .parquet file which can be converted into a
           pd.DataFrame, containing at least two columns, one with the sentences in the corpus to be searched, and one
           with their respective embeddings. The model used to embed such sentences must be the same as the one given
           in model_name.
    :param expected_embeddings_colname: the column name, in the pd.DataFrame mentioned above, containing the embeddings.
    :param model_name: the model to use to embed the user query; available options: 'Word2Vec',
           'Word2Vec_TfIdf_weighted', 'ELMo', 'BERT'.
    :param model: the trained model used to embed the user query. It can be an instance of Word2Vec, ElmoEmbedder or
           SentenceTransformer, or a string pointing to a .pickle file representing the trained model of choice, to be
           loaded. This must be the same model chosen in model_name. Defaults: None. If None, the standard pre-trained
           ELMo or BERT models will be used. A model entry must be given if model_name is 'Word2Vec' or
           'Word2Vec_TfIdf_weighted', or else a FileNotFoundError will be raised.
    :param expected_sentence_colname: the column name, in the pd.DataFrame loaded from the .parquet file in
           trained_df_path, containing the sentences.
    :param distance_metric: the metric used to compare the user query embedding and the corpus embeddings and calculate
           the semantic similarity. Default: cosine similarity.
    :param metric_colname: the name of the column to be added to the trained_df containing the distance between the
           user input embedding and each sentence embedding, calculated using the distance_metric.
    :param tfidf_vectorizer: a TfidfVectorizer object, or a string pointing to a .pickle file representing a
           TfidfVectorizer object. Default: None. This only needs to be provided when model_name is
           'Word2Vec_TfIdf_weighted'.
    :return: user_input_embedding: the embedding of the user_input
             trained_df: a pd.DataFrame, which is the same as the one extracted from the .parquet file in
             trained_df_path, with an added column representing the distance between the user input embedding and each
             sentence embedding, calculated using the distance_metric.
    """

    if not user_input.strip():
        raise KeyError('User input cannot be empty.')
    assert model_name in ['Word2Vec', 'Word2Vec_TfIdf_weighted', 'ELMo', 'BERT'], \
        "Model name not recognized. Please choose one of ['Word2Vec', 'Word2Vec_TfIdf_weighted', 'ELMo', 'BERT']."
    assert isinstance(trained_df, pd.DataFrame) or '.parquet' in trained_df, \
        "trained_df_path should be a pd.DataFrame, or a string pointing to a .parquet file. Format not recognised."
    if '.parquet' in trained_df:
        logger.info(f'Loading DataFrame from {trained_df} file.')
        trained_df = pd.read_parquet(trained_df)
    assert expected_sentence_colname in trained_df.columns, \
        f"Please check your input trained DataFrame. It should contain a '{expected_sentence_colname}' column."
    assert expected_embeddings_colname in trained_df.columns, \
        f"Please check your input trained DataFrame. It should contain a '{expected_embeddings_colname}' column."
    assert model is None or isinstance(model, (Word2Vec, ElmoEmbedder, SentenceTransformer)) or '.pickle' in model, \
        "model should be None, or a string pointing to a .pickle file, or an instance of Word2Vec, ElmoEmbedder or" \
        "SentenceTransformer. Other file formats are not recognised."
    assert tfidf_vectorizer is None or isinstance(tfidf_vectorizer, TfidfVectorizer) or '.pickle' in tfidf_vectorizer,\
        "tfidf_vectorizer_path should be None, or a string pointing to a .pickle file, or an instance of" \
        "TfidfVectorizer. Other file formats are not recognised."

    if isinstance(model, str) and ".pickle" in model:
        logger.info(f'Loading model from {model} file.')
        model = pickle.load(open(model, "rb"))
        assert isinstance(model, (Word2Vec, ElmoEmbedder, SentenceTransformer)), \
            "The .pickle provided does not correspond to a Word2Vec, ElmoEmbedder or SentenceTransformer model."

    logger.info(f'Embedding user query using the {model_name} model...')
    if model_name in ['Word2Vec', 'Word2Vec_TfIdf_weighted']:
        if model is None:
            raise FileNotFoundError(
                "When using 'Word2Vec' or 'Word2Vec_TfIdf_weighted' model_name, the model variable must be provided "
                "and it cannot be left empty."
            )
        vocabs = [model.wv.vocab]
        word_to_tfidf_score = dict()
        tok_input = nltk.word_tokenize(user_input)

        if model_name == 'Word2Vec_TfIdf_weighted':
            assert tfidf_vectorizer is not None, \
                "When using the 'Word2Vec_TfIdf_weighted' model you must provide tfidf_vectorizer_path as a str " \
                "pointing to a .pickle."
            if isinstance(tfidf_vectorizer, str) and '.pickle' in tfidf_vectorizer:
                logger.info(f'Loading vectorizer from {tfidf_vectorizer} file.')
                tfidf_vectorizer = pickle.load(open(tfidf_vectorizer, "rb"))
            word_to_tfidf_score = dict(zip(tfidf_vectorizer.get_feature_names(), tfidf_vectorizer.idf_))
            vocabs.append(word_to_tfidf_score)

        words_embeddings = [
            model.wv.get_vector(word) * word_to_tfidf_score.get(word, 1)
            for word in tok_input
            if all(word in vocab for vocab in vocabs)
        ]
        if len(words_embeddings) == 0:
            logger.error(
                'None of the words inputted are in the Word2Vec vocabulary. Please change your input or try a '
                'different model, such as ELMo or BERT. Returning empty array and DataFrame.'
            )
            return np.array([]), pd.DataFrame()

        ignored_words = [word for word in tok_input if not all(word in vocab for vocab in vocabs)]
        # vocabulary is case sensitive. Need to keep case if word exists in both cases; if not,
        # need to lowercase/uppercase as appropriate to avoid a useful word being excluded. Not done yet.
        if len(ignored_words) != 0:
            logger.warning(
                f'The following words are not in the trained vocabulary and were therefore excluded from the '
                f'search: {ignored_words}'
            )

        user_input_embedding = np.mean(words_embeddings, axis=0)

    if model_name == 'ELMo':
        tok_input = nltk.word_tokenize(user_input)
        if model is None:
            logger.warning(
                'No ELMo model was provided. Defaulting to using an instance of allennlp.commands.elmo.ElmoEmbedder '
                'to embed the user query.'
            )
            model = ElmoEmbedder()
        words_embeddings = model.embed_sentence(tok_input)  # shape (3, len(tok_input), 1024)
        user_input_embedding = np.average(words_embeddings, axis=1)[2]

    if model_name == 'BERT':
        if model is None:
            logger.warning(
                'No BERT model was provided. Defaulting to using an instance of sentence_transformers.'
                'SentenceTransformer to embed the user query, with model \'bert-base-nli-stsb-mean-tokens\'.'
            )
            model = SentenceTransformer('bert-base-nli-stsb-mean-tokens')
        user_input_embedding = model.encode([user_input])[0]
    logger.info('User query has been embedded.')

    trained_df[metric_colname] = cdist(
        list(trained_df[expected_embeddings_colname]), [user_input_embedding], metric=distance_metric
    )
    return user_input_embedding, trained_df


import numpy as np
import os
import pandas as pd
import logging
import nltk
import pickle
from gensim.models import Word2Vec
from sklearn.feature_extraction.text import TfidfVectorizer
from allennlp.commands.elmo import ElmoEmbedder
from sentence_transformers import SentenceTransformer
import multiprocessing
from typing import List, Tuple, Dict, Optional, Union


logger = logging.getLogger(__name__)


class Embedder:
    """
    This class provides methods for calculating sentence-level embeddings given a list of sentences (not tokenised)
    using Word2Vec (with an option to weight the embeddings by their tf-idf score), ELMo and BERT. It also provides
    static methods to save the embeddings and the models, and to add the embeddings as a column of a pd.DataFrame
    (useful if the DataFrame already contains another column in which each row correspond to one sentence in the list
    of sentences that have been embedded, in the same order, so one can quickly match each sentence with its embedding).
    Note that Word2Vec is the only model that is trained only on the list of sentences provided. For ELMo and BERT,
    we are making use of pre-trained models.
    """
    def __init__(self, list_of_sentences: List[str]) -> None:
        """
        :param list_of_sentences: a list where each item is a string containing a full sentence.
        """
        self.list_of_sentences = list_of_sentences
        logger.info(f'Initializing Embedder class with {len(self.list_of_sentences)} sentences.')
        assert_message = 'list_of_sentences must be a list of strings, where each string is a full sentence.'
        assert isinstance(self.list_of_sentences, list), assert_message
        assert all(isinstance(item, str) for item in self.list_of_sentences), assert_message

    def compute_word2vec_embeddings(
            self,
            tfidf_weights: bool = True,
            embedding_dim: int = 300,
            window: int = 5,
            min_count: int = 0,
            negative: int = 15,
            iterations: int = 10,
            workers: int = multiprocessing.cpu_count(),
            **kwargs
    ) -> Tuple[np.ndarray, Word2Vec, Optional[TfidfVectorizer]]:
        """
        Calculates the sentence-level embeddings of the sentences in list_of_sentences, using Word2Vec. If tfidf_weights
        is True, the word-level embeddings are multiplied with their tfidf score before calculating sentence-level
        embeddings.
        :param tfidf_weights: boolean. If True, the word embeddings are weighted with their tf-idf score.
        :param embedding_dim:
        :param window:
        :param min_count:
        :param negative:
        :param iterations:
        :param workers:
        :return: sentence_embeddings: a np.array containing sentence-level embeddings for each sentence in
                 list_of_sentences.
                 w2v: the Word2Vec model trained using list_of_sentences
                 tfidf_vectorizer: the TfidfVectorizer model. If tfidf_weights is False, this deafults to None.
        """

        tok_sentences = [nltk.word_tokenize(sentence) for sentence in self.list_of_sentences]
        # list of lists of strings (words)
        logger.info('Initializing Word2Vec.')
        w2v = Word2Vec(
            tok_sentences, size=embedding_dim, window=window, min_count=min_count, negative=negative, iter=iterations,
            workers=workers, **kwargs
        )
        word_to_tfidf_scores = dict()
        list_of_vocabs = [w2v.wv.vocab]
        tfidf_vectorizer = None

        if tfidf_weights:
            word_to_tfidf_scores, tfidf_vectorizer = self._compute_tfidf_weights(w2v)
            list_of_vocabs.append(word_to_tfidf_scores)

        sentence_embeddings = []

        for sentence in tok_sentences:
            words_embeddings = [
                w2v.wv.get_vector(word) * word_to_tfidf_scores.get(word, 1)
                for word in sentence if all([word in vocab for vocab in list_of_vocabs])
            ]
            # if statement is to exclude words that appear less than 'min_count' times in the w2v.wv.vocab (see Word2Vec
            # model) and, if tfidf_weights is True, also to exclude words that do not appear in word_to_tfidf_scores (to
            # note that this tfidf vocabulary is smaller than w2v.wv.vocab so some additional words are lost).

            mean_sentence_vector = np.mean(words_embeddings, axis=0)
            sentence_embeddings.append(mean_sentence_vector)
        sentence_embeddings = np.array(sentence_embeddings)  # shape (len(tok_sentences), embedding_dim)

        return sentence_embeddings, w2v, tfidf_vectorizer

    def _compute_tfidf_weights(self, w2v: Word2Vec) -> Tuple[Dict[str, float], TfidfVectorizer]:
        """
        Compute the tfidf weights given a list of sentences, if the words are in the given Word2Vec instance's
        vocabulary.
        :param w2v: a Word2Vec object (instance). Words that are not in the Word2Vec vocabulary will be ignores
        :return: word_to_tfidf_score: a dictionary with keys being the words in tf_idf_vectorizer (str) and the values
                                      being their corresponding tfidf scores (float).
                 tfidf_vectorizer: the TfidfVectorizer object, available for future re-use
        """
        logger.info('Initializing TfidfVectorizer.')
        tfidf_vectorizer = TfidfVectorizer(lowercase=False, vocabulary=list(w2v.wv.vocab))
        # forcing the w2v vocabulary, as otherwise the tf-idf generated vocabulary will not match completely.
        tfidf_vectorizer.fit(self.list_of_sentences)
        word_to_tfidf_score = dict(zip(tfidf_vectorizer.get_feature_names(), tfidf_vectorizer.idf_))
        # Alternative 1:
        # word_to_tfidf_score = {
        #     item[0]: tfidf_vectorizer.idf_[item[1]] for item in tfidf_vectorizer.vocabulary_.items()
        # }
        # Alternative 2:
        # word_to_tfidf_score = defaultdict(lambda: max(tfidf_vectorizer.idf_),
        #    [(w, tfidf_vectorizer.idf_[i]) for w, i in tfidf_vectorizer.vocabulary_.items()])

        return word_to_tfidf_score, tfidf_vectorizer

    def compute_elmo_embeddings(
            self,
            options_file: str = "https://allennlp.s3.amazonaws.com/models/elmo/2x4096_512_2048cnn_2xhighway/"
                                "elmo_2x4096_512_2048cnn_2xhighway_options.json",
            weight_file: str = "https://allennlp.s3.amazonaws.com/models/elmo/2x4096_512_2048cnn_2xhighway/"
                               "elmo_2x4096_512_2048cnn_2xhighway_weights.hdf5"
    ) -> Tuple[np.ndarray, ElmoEmbedder]:
        """
        Calculates the ELMo embeddings of the sentences within the corpus. Each embedding has 3 layers and 1024
        dimensions.

        :param options_file: from ELMo, the default options for the model
        :param weight_file: from ELMo, the pre-trained weights for the model
        :return: sentence_embeddings: a np.array containing sentence-level embeddings for each sentence in
                 list_of_sentences
                 elmo: the ElmoEmbedder model used to embed list_of_sentences
        """

        tok_sentences = [nltk.word_tokenize(sentence) for sentence in self.list_of_sentences]
        logger.info('Initializing ElmoEmbedder.')
        logger.debug(f'Parameters: \n options_file: {options_file} \n weight_file: {weight_file}')
        elmo = ElmoEmbedder(options_file, weight_file)

        word_embeddings = elmo.embed_sentences(tok_sentences, batch_size=16)  # this returns a generator such that
        # len(list(embedding_iter)) = len(tok_sentences)

        sentence_embeddings = []
        for embedding in word_embeddings:
            sentence_embedding = np.mean(embedding, axis=1)[2]  # taking last layer (ELMo layer is number 3, hence [2]).
            sentence_embeddings.append(sentence_embedding)
        sentence_embeddings = np.array(sentence_embeddings)  # shape (len(tok_sentences), 1024)

        return sentence_embeddings, elmo

    def compute_bert_embeddings(self, model: str) -> Tuple[np.ndarray, SentenceTransformer]:
        """
        Calculates the BERT embeddings of the sentences within the corpus. Each embedding has 768 dimensions.
        :param model: a string containing the BERT model to use. For a list of available models, see
               https://github.com/UKPLab/sentence-transformers#pretrained-models
        :return: sentence_embeddings: a np.array containing sentence-level embeddings for each sentence in
                 list_of_sentences.
                 bert: the SentenceTransformer BERT model used to embed list_of_sentences
        """
        logger.info('Initializing SentenceTransformer.')
        logger.debug(f'Model used is {model}.')
        bert = SentenceTransformer(model)
        sentence_embeddings = bert.encode(self.list_of_sentences)
        sentence_embeddings = np.array(sentence_embeddings)  # shape (len(self.list_of_sentences), 768)

        return sentence_embeddings, bert

    @staticmethod
    def save_embeddings(embeddings: np.ndarray, folder: str, filename: str) -> None:
        assert '.npy' in filename, "Please ensure you are using a .npy format for saving the array with the embeddings."
        np.save(os.path.join(folder, filename), embeddings)

    @staticmethod
    def save_model(
            model: Union[Word2Vec, TfidfVectorizer, ElmoEmbedder, SentenceTransformer],
            folder: str,
            filename: str
    ) -> None:
        assert '.pickle' in filename, "Please ensure you are using a .pickle format for saving your model."
        pickle.dump(model, open(os.path.join(folder, filename), "wb"))

    @staticmethod
    def add_embeddings_to_corpus_df(
            input_path: Union[str, pd.DataFrame],
            sentence_embeddings: Union[str, np.ndarray],
            embeddings_column_name: str
    ) -> pd.DataFrame:
        """
        Add the given sentence-level embeddings (as an extra column) to a pd.DataFrame containing at least a column
        where each rows represents one sentence. The order (and length) of the sentences and the embeddings must be the
        same.
        :param input_path: the path to the source pd.DataFrame. Can be a pd.DataFrame directly, or a str pointing to
        a .parquet or .cvs file. No other formats are accepted.
        :param sentence_embeddings: path to the embeddings to be added. Can be a np.array or a str pointing to
        a .npy file. No other formats are accepted.
        :param embeddings_column_name: the column name of the added column containing the embeddings.
        :return: df: a copy of the pd.DataFrame specified in input_path with the added column containing the embeddings.
        """
        if isinstance(input_path, str) and ".parquet" in input_path:
            logger.info(f'Loading DataFrame from {input_path} file.')
            df = pd.read_parquet(input_path)
        elif isinstance(input_path, str) and ".csv" in input_path:
            logger.info(f'Loading DataFrame from {input_path} file.')
            df = pd.read_csv(input_path)
        elif isinstance(input_path, pd.DataFrame):
            df = input_path
        else:
            raise TypeError(
                "Please ensure your 'input path' is either a string pointing to either a .parquet or a .csv file, or a "
                "pandas.DataFrame. Other input formats are not recognised by this method."
            )

        if isinstance(sentence_embeddings, str) and ".npy" in sentence_embeddings:
            logger.info(f'Loading np.array from {sentence_embeddings} file.')
            sentence_embeddings = np.load(sentence_embeddings)
        elif isinstance(sentence_embeddings, np.ndarray):
            pass  # sentence_embeddings = sentence_embeddings
        else:
            raise TypeError(
                "Please ensure your 'sentence_embeddings' is either a .npy file or a numpy.array. Other input formats "
                "are not recognised by this method."
            )
        assert df.shape[0] == len(sentence_embeddings), \
            f"pd.DataFrame and embeddings could not be matched with lengths {df.shape[0]} and " \
            f"{len(sentence_embeddings)} respectively."

        df[embeddings_column_name] = list(sentence_embeddings)  # df expects a list input, not np.array
        return df

    @staticmethod
    def df_to_parquet(df: pd.DataFrame, output_filename: str) -> None:
        assert '.parquet' in output_filename, "Please ensure you are using a .parquet format for saving your model."
        df.to_parquet(output_filename + '.gzip', compression='gzip')



try:
    from pdf2embeddings import scraper, arrange_text, embedder, process_user_queries
except ModuleNotFoundError as err:
    print(f"You have not imported all the required packages. \n{err}")
__version__ = '0.2.0'



import json
import os


def create_text_cleaning_json(folder: str) -> None:
    to_replace = {
        "\n": "",
        "Cat.": "Category",
        "cat.": "category",
        "Mr.": "Mr",
        "Mrs.": "Mrs",
        "Dr.": "Dr",
        "e.g.": "eg",
        "Op.": "Operational",
        "Cap.": "Capacity",
        "�": "",
        "\x0c": ""
    }
    with open(os.path.join(folder, 'words_to_replace.json'), 'w') as f:
        json.dump(to_replace, f)


def create_filenames_json(folder: str) -> None:
    filenames = {
        'Word2Vec': {
            "embeddings_filename": "w2v_embeddings.npy",
            "model_filename": "word2vec.pickle",
            "column_name": "Word2Vec",
            "parquet_filename": "corpus_by_sentence_with_Word2Vec.parquet"
        },
        'Word2Vec_tfidf_weighted': {
            "embeddings_filename": "w2v_with_tfidf_embeddings.npy",
            "model_filename": "word2vec_tfidf.pickle",
            "vectorizer_filename": "tfidf_vectorizer.pickle",
            "column_name": "Word2Vec_with_TfIdf_weights",
            "parquet_filename": "corpus_by_sentence_with_Word2Vec_TfIdf_weighted.parquet"
        },
        'ELMo': {
            "embeddings_filename": "elmo_embeddings.npy",
            "model_filename": "elmo_model.pickle",
            "column_name": "ELMo_layer_3",
            "parquet_filename": "corpus_by_sentence_with_ELMo.parquet"
        },
        'BERT': {
            "embeddings_filename": "bert_embeddings.npy",
            "model_filename": "bert_model.pickle",
            "column_name": "BERT",
            "parquet_filename": "corpus_by_sentence_with_BERT.parquet"
        }
    }
    with open(os.path.join(folder, 'filenames.json'), 'w') as f:
        json.dump(filenames, f)


if __name__ == '__main__':
    CONFIG_DIR = os.getenv('CONFIG_DIR')
    create_text_cleaning_json(CONFIG_DIR)
    create_filenames_json(CONFIG_DIR)



import pandas as pd
import numpy as np
import nltk
import logging


logger = logging.getLogger(__name__)


class CorpusGenerator:
    """
    This class contains methods for converting a "horizontal" pd.DataFrame or scraped text from files, where:
        - each column contains the text of a separate pdf file, and
        - each row within a column contains the text of one page within that pdf
    into a new "vertical" pd.DataFrame with only 3 columns:
        - 'sentence': one row per each sentence in the full corpus
        - 'pdf_name': the name of the PDF file each sentence is taken from
        - 'page_number': the page number where the sentence can be found within the file 'pdf_name'.
           NOTE: sentences that span across multiple pages have been split so no sentence can run across the next page.
    """
    def __init__(self, df: pd.DataFrame) -> None:
        """
        :param df: the input pd.DataFrame as described in class docstring.
        """
        self.df = df

    def df_by_page_to_df_by_sentence(self) -> pd.DataFrame:
        """
        :return: df_by_sentence: the output pd.DataFrame as described in class docstring.
        """
        logger.info('Starting DataFrame conversion...')
        full_corpus_list = [
            {'sentence': sentence,
             'pdf_name': pdf_name,
             'page_number': page_i + 1}
            for pdf_name in self.df.columns
            for page_i, text in enumerate(self.df[pdf_name]) if isinstance(text, str)
            for sentence in nltk.sent_tokenize(text)
        ]
        # isinstance(text, str) is required to handle np.nan values: when a PDF has less pages than the longest
        # PDF, all rows corresponding to pages after the last page of that PDF are filled with np.nan.
        df_by_sentence = pd.DataFrame(full_corpus_list)
        df_by_sentence['sentence'].replace('', np.nan, inplace=True)
        df_by_sentence.dropna(axis=0, how='any', inplace=True)
        # this gets rid of lots of empty sentences when original file contains multiple dots '.....' (e.g. in table
        # content), which are empty because for every '.' a new sentence is generated.
        logger.info('New DataFrame created.')
        return df_by_sentence



import os
from setuptools import find_packages, setup
from pdf2embeddings.__init__ import __version__


def read(*paths):
    """Builds a file path from *paths and returns the content."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()


setup(
    name='pdf2embeddings',
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    version=__version__,
    description='NLP tool for scraping text from a corpus of PDF files, embedding the sentences in the text and '
                'finding semantically similar sentences to a given search query.',
    long_description=read("README.md"),
    long_description_content_type='text/markdown',
    url='https://github.com/moj-analytical-services/airflow-pdf2embeddings',
    download_url='https://pypi.python.org/pypi/pdf2embeddings',
    tests_require=["pytest", "flaky"],
    include_package_data=True,
    python_requires=">=3.6",
    license='MIT',
    install_requires=[
        "allennlp==0.9.0",
        "gensim==3.8.1",
        "nltk==3.4.5",
        "numpy==1.18.2",
        "pandas==0.25.3",
        "ply==3.11",
        "pyarrow==0.16.0",
        "pytest==5.4.1",
        "scikit-learn==0.22.1",
        "scipy==1.4.1",
        "sentence-transformers==0.2.5.1",
        "slate3k==0.5.3",
        "typing==3.7.4.1",
        "tqdm==4.45.0",
        "PyYAML~=5.1.2",
        "setuptools~=47.1.1",
        "s3fs~=0.4.2",
        "sklearn~=0.0"
    ],
    author='moj-analytical-services',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ]
)
